/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.sample.common

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.snackbar.BaseTransientBottomBar
import com.google.android.material.snackbar.Snackbar
import com.kakao.sdk.partner.sample.common.internal.fromChat
import com.kakao.sdk.partner.talk.chatList
import com.kakao.sdk.partner.talk.chatMembers
import com.kakao.sdk.partner.talk.friendsForPartner
import com.kakao.sdk.partner.talk.model.FriendFilter
import com.kakao.sdk.partner.talk.model.PartnerFriend
import com.kakao.sdk.partner.talk.model.PartnerFriendsContext
import com.kakao.sdk.partner.talk.sendCustomMessageForPartner
import com.kakao.sdk.partner.talk.sendCustomMessageToChats
import com.kakao.sdk.partner.talk.sendDefaultMessageForPartner
import com.kakao.sdk.partner.talk.sendDefaultMessageToChats
import com.kakao.sdk.partner.talk.sendScrapMessageForPartner
import com.kakao.sdk.partner.talk.sendScrapMessageToChats
import com.kakao.sdk.partner.user.ageAuthInfo
import com.kakao.sdk.partner.user.createShippingAddress
import com.kakao.sdk.partner.user.createShippingAddress
import com.kakao.sdk.partner.user.meForPartner
import com.kakao.sdk.partner.user.model.AgeAuthLevel
import com.kakao.sdk.partner.user.model.AgeCriteria
import com.kakao.sdk.partner.user.signupForPartner
import com.kakao.sdk.partner.user.updateShippingAddress
import com.kakao.sdk.partner.user.updateShippingAddress
import com.kakao.sdk.partner.user.upgradeScopes
import com.kakao.sdk.partner.user.verifyAge
import com.kakao.sdk.sample.common.OpenFragment
import com.kakao.sdk.sample.common.internal.ApiAdapter
import com.kakao.sdk.sample.common.internal.FriendsActivity
import com.kakao.sdk.sample.common.internal.Log
import com.kakao.sdk.sample.common.internal.PickerItem
import com.kakao.sdk.talk.TalkApiClient
import com.kakao.sdk.talk.model.FriendOrder
import com.kakao.sdk.talk.model.Friends
import com.kakao.sdk.talk.model.Order
import com.kakao.sdk.user.UserApiClient

/**
 * A simple [Fragment] subclass.
 */
class PartnerFragment : Fragment() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let { templateIds = it }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_partner, container, false)
    }

    private fun showSnackbar(any: Any) {
        val snackbar =
            Snackbar.make(requireView(), any.toString(), BaseTransientBottomBar.LENGTH_LONG)
        val layout = snackbar.view as Snackbar.SnackbarLayout
        layout.minimumHeight = 100
        snackbar.show()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewManager = LinearLayoutManager(requireContext())
        viewAdapter = ApiAdapter(
            listOf(
                ApiAdapter.Item.Header("Auth API"),
                ApiAdapter.Item.ApiItem("verifyAge()") {
                    UserApiClient.instance.verifyAge(requireContext()) { error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        } else {
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("verifyAge(1차 인증)") {
                    UserApiClient.instance.verifyAge(
                        requireContext(),
                        authLevel = AgeAuthLevel.AUTH_LEVEL1
                    ) { error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        } else {
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("verifyAge(2차 인증)") {
                    UserApiClient.instance.verifyAge(
                        requireContext(),
                        authLevel = AgeAuthLevel.AUTH_LEVEL2
                    ) { error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        } else {
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }
                    }
                },
                ApiAdapter.Item.Header("User API"),
                ApiAdapter.Item.ApiItem("meForPartner()") {
                    UserApiClient.instance.meForPartner { user, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("$user")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("signupForPartner()") {
                    UserApiClient.instance.signupForPartner { error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("signup 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("ageAuthInfo() - ageLimit - YEAR") {
                    UserApiClient.instance.ageAuthInfo(ageLimit = 29, ageCriteria = AgeCriteria.YEAR) { info, error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 조회 실패", error)
                        } else {
                            Log.i(TAG, info.toString())
                            showSnackbar(info.toString())
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("ageAuthInfo() - ageLimit - INTERNATIONAL") {
                    UserApiClient.instance.ageAuthInfo(ageLimit = 29, ageCriteria = AgeCriteria.INTERNATIONAL) { info, error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 조회 실패", error)
                        } else {
                            Log.i(TAG, info.toString())
                            showSnackbar(info.toString())
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("ageAuthInfo()") {
                    UserApiClient.instance.ageAuthInfo { info, error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 조회 실패", error)
                        } else {
                            Log.i(TAG, info.toString())
                            showSnackbar(info.toString())
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("upgradeScopes()") {
                    val scopes = listOf("birthday")
                    UserApiClient.instance.upgradeScopes(scopes) { info, error ->
                        if (error != null) {
                            Log.e(TAG, "Upgrade Scopes 실패", error)
                        } else {
                            Log.i(TAG, info.toString())
                            showSnackbar(info.toString())
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("createShippingAddress") {
                    UserApiClient.instance.createShippingAddress(requireContext()) { addressId, error ->
                        if (error != null) {
                            Log.i(TAG, "배송지 추가 실패 $error")
                            showSnackbar("배송지 추가 실패 $error")
                            return@createShippingAddress
                        }

                        Log.i(TAG, "배송지 추가 성공 $addressId")
                        showSnackbar("배송지 추가 성공 $addressId")
                    }
                },
                ApiAdapter.Item.ApiItem("updateShippingAddress") {
                    UserApiClient.instance.selectShippingAddress(requireContext()) { addressId, error ->
                        if (error != null) {
                            Log.i(TAG, "배송지 선택 실패 $error")
                            showSnackbar("배송지 선택 실패 $error")
                            return@selectShippingAddress
                        }

                        UserApiClient.instance.updateShippingAddress(
                            requireContext(),
                            addressId!!
                        ) { updatedAddressId, err ->
                            if (err != null) {
                                Log.i(TAG, "배송지 수정 실패 $err")
                                showSnackbar("배송지 수정 실패 $err")
                                return@updateShippingAddress
                            }

                            Log.i(TAG, "배송지 수정 성공 $updatedAddressId")
                            showSnackbar("배송지 수정 성공 $updatedAddressId")
                        }
                    }
                },
                ApiAdapter.Item.Header("Friend Picker API"),
                ApiAdapter.Item.ApiItem("show friend picker") {
                    Intent(context, FriendPickerSettingActivity::class.java).run {
                        startActivity(this)
                    }
                },
                ApiAdapter.Item.ApiItem("show chat picker") {
                    Intent(context, ChatPickerSettingActivity::class.java).run {
                        startActivity(this)
                    }
                },
                ApiAdapter.Item.ApiItem("show tab picker") {
                    Intent(context, TabPickerSettingActivity::class.java).run {
                        startActivity(this)
                    }
                },
                ApiAdapter.Item.Header("Talk API"),
                ApiAdapter.Item.ApiItem("friendsForPartner()") {
                    TalkApiClient.instance.friendsForPartner { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(friends!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("friendsForPartner(context:) - reculsive") {
                    var nextFriendsContext = PartnerFriendsContext(
                        offset = 0,
                        limit = 10,
                        order = Order.DESC,
                        countryCodes = listOf("kr", "us"),
                    )

                    recursiveAppFriendsCompletion =
                        recursiveAppFriendsCompletion@{ nextFriends, err ->
                            if (err == null) {
                                if (nextFriends != null) {
                                    try {
                                        nextFriends.afterUrl?.let {
                                            nextFriendsContext = PartnerFriendsContext(url = it)
                                        } ?: return@recursiveAppFriendsCompletion

                                    } catch (e: IllegalArgumentException) {
                                        return@recursiveAppFriendsCompletion
                                    }
                                }

                                TalkApiClient.instance.friendsForPartner(context = nextFriendsContext) { friends, error ->
                                    if (error != null) {
                                        Log.e(TAG, "카카오톡 친구 목록 받기 실패", error)
                                    } else if (friends != null) {
                                        Log.i(
                                            TAG,
                                            "카카오톡 친구 목록 받기 성공 \n${friends.elements?.joinToString("\n")}"
                                        )
                                        recursiveAppFriendsCompletion?.let { it(friends, null) }
                                    }
                                }
                            }
                        }

                    recursiveAppFriendsCompletion?.let { it(null, null) }
                },
                ApiAdapter.Item.ApiItem("friendsForPartner(context:) - PartnerFriendsContext") {
                    TalkApiClient.instance.friendsForPartner(
                        context = PartnerFriendsContext(
                            offset = 0,
                            limit = 3,
                            order = Order.DESC,
                            friendOrder = FriendOrder.NICKNAME,
                            friendFilter = FriendFilter.REGISTERED,
                            countryCodes = listOf("kr", "us")
                        )
                    ) { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(friends!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("chatList()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(chats!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("chatMembers()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (chats != null) {

                            chats.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("채팅방이 없습니다.")
                                } else {

                                    val chatId = it.first().id

                                    TalkApiClient.instance.chatMembers(chatId) { chatMembers, error ->
                                        if (error != null) {
                                            showSnackbar(error)
                                        } else {
                                            showSnackbar(chatMembers!!)
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendCustomMessageForPartner()") {
                    TalkApiClient.instance.friendsForPartner { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (friends != null) {
                            friends.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 친구가 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        requireContext(),
                                        it.map { friend ->
                                            PickerItem(
                                                friend.uuid,
                                                friend.profileNickname ?: "",
                                                friend.profileThumbnailImage
                                            )
                                        }
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val templateId = templateIds.getLong("customMessage")

                                        TalkApiClient.instance.sendCustomMessageForPartner(
                                            selectedItems,
                                            templateId
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendScrapMessageForPartner()") {
                    TalkApiClient.instance.friendsForPartner { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (friends != null) {
                            friends.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 친구가 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        requireContext(),
                                        it.map { friend ->
                                            PickerItem(
                                                friend.uuid,
                                                friend.profileNickname ?: "",
                                                friend.profileThumbnailImage
                                            )
                                        }
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val url = "https://developers.kakao.com"

                                        TalkApiClient.instance.sendScrapMessageForPartner(
                                            selectedItems,
                                            url
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendDefaultMessageForPartner()") {
                    TalkApiClient.instance.friendsForPartner { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (friends != null) {
                            friends.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 친구가 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        requireContext(),
                                        it.map { friend ->
                                            PickerItem(
                                                friend.uuid,
                                                friend.profileNickname ?: "",
                                                friend.profileThumbnailImage
                                            )
                                        }
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val template = OpenFragment.defaultLocation

                                        TalkApiClient.instance.sendDefaultMessageForPartner(
                                            selectedItems,
                                            template
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendCustomMessageToChats()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (chats != null) {
                            chats.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 채팅방이 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        requireContext(),
                                        it.map(PickerItem.Companion::fromChat)
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val templateId = templateIds.getLong("customMessage")

                                        TalkApiClient.instance.sendCustomMessageToChats(
                                            selectedItems.map { item -> item.toLong() },
                                            templateId
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendDefaultMessageToChats()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (chats != null) {
                            chats.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 채팅방이 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        requireContext(),
                                        it.map(PickerItem.Companion::fromChat)
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val template = OpenFragment.defaultFeed

                                        TalkApiClient.instance.sendDefaultMessageToChats(
                                            selectedItems.map { item -> item.toLong() },
                                            template
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }

                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendScrapMessageToChats()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (chats != null) {
                            chats.elements?.let { chat ->
                                if (chat.isEmpty()) {
                                    showSnackbar("메시지 보낼 채팅방이 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        requireContext(),
                                        chat.map(PickerItem::fromChat)
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val url = "https://developers.kakao.com"

                                        TalkApiClient.instance.sendScrapMessageToChats(
                                            selectedItems.map { it.toLong() },
                                            url
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                })
        )

        view.findViewById<RecyclerView>(R.id.recyclerView).apply {
            setHasFixedSize(true)
            addItemDecoration(
                DividerItemDecoration(context, DividerItemDecoration.VERTICAL)
            )
            layoutManager = viewManager
            adapter = viewAdapter
        }
    }

    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: RecyclerView.LayoutManager

    private lateinit var templateIds: Bundle


    private var recursiveAppFriendsCompletion: ((Friends<PartnerFriend>?, Error?) -> Unit)? = null

    companion object {
        const val TAG = "KakaoSDKSample"

        fun newInstance(templateIds: Bundle): PartnerFragment =
            PartnerFragment().apply {
                arguments = templateIds
            }
    }
}
