/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.sample.common

import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.friend.core.PickerClient
import com.kakao.sdk.friend.core.model.DisableSelectOption
import com.kakao.sdk.friend.core.model.DisableSelectReason
import com.kakao.sdk.friend.core.model.PickerChatFilter
import com.kakao.sdk.friend.core.model.PickerChatSelectionType
import com.kakao.sdk.friend.core.model.PickerFriendFilter
import com.kakao.sdk.friend.core.model.PickerOrientation
import com.kakao.sdk.friend.core.model.PickerUsingOsFilter
import com.kakao.sdk.friend.core.model.SelectedChat
import com.kakao.sdk.friend.core.model.SelectedUsers
import com.kakao.sdk.friend.core.model.ViewAppearance
import com.kakao.sdk.partner.friend.client.select
import com.kakao.sdk.partner.friend.client.selectPopup
import com.kakao.sdk.partner.friend.model.PickerTabRequestParams
import com.kakao.sdk.partner.sample.common.databinding.ActivityTabPickerSettingBinding

class TabPickerSettingActivity : AppCompatActivity() {
    private lateinit var binding: ActivityTabPickerSettingBinding
    private var title: String? = null

    private var friendFilter: PickerFriendFilter? = null
    private var chatFilter: List<PickerChatFilter>? = null
    private var viewAppearance: ViewAppearance? = null
    private var countryCodes: List<String>? = null
    private var osFilter: PickerUsingOsFilter? = null
    private var orientation: PickerOrientation? = null
    private var disableSelectOptions: List<DisableSelectOption>? = null
    private var maxPickableCount: Int = 30
    private var minPickableCount: Int = 1

    private val disableCustoms = mutableListOf<DisableSelectOption>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityTabPickerSettingBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val friendFilterArray = resources.getStringArray(R.array.friend_filter)
        val deviceOsArray = resources.getStringArray(R.array.device_os)
        val orientationArray = resources.getStringArray(R.array.picker_orientation)
        val viewAppearanceArray = resources.getStringArray(R.array.view_appearance)

        binding.apply {
            disableSelectClearBtn.setOnClickListener {
                disableCustoms.clear()
                disableCustomCount.text = "${disableCustoms.size}"
            }

            disableSelectAddBtn.setOnClickListener {
                val message = disableMessageEt.text.toString()
                val uuidText = uuidEt.text.toString()
                val uuids = if (uuidText.isEmpty()) {
                    null
                } else {
                    uuidText.split(",").map { it.trim() }.toList()
                }
                disableCustoms.add(DisableSelectOption(DisableSelectReason.CUSTOM, message, uuids))

                disableMessageEt.text.clear()
                uuidEt.text.clear()
                disableCustomCount.text = "${disableCustoms.size}"
            }

            cancelBtn.setOnClickListener {
                finish()
            }

            createBtn.setOnClickListener {
                title = getPickerTitle()
                maxPickableCount = getMaxPickableCount()
                minPickableCount = getMinPickableCount()
                countryCodes = getCountryCodes()
                friendFilter = getPickerFriendFilter(friendFilterArray)
                chatFilter = getPickerChatFilter()
                osFilter = getOsFilter(deviceOsArray)
                viewAppearance = getViewAppearance(viewAppearanceArray)
                orientation = getPickerOrientation(orientationArray)
                disableSelectOptions = getDisableSelect()

                val callback =
                    { selectedUsers: SelectedUsers?, selectedChat: SelectedChat?, error: Throwable? ->
                        if (error != null) {
                            SdkLog.e("정보 불러오기 실패 $error")
                            Toast.makeText(
                                this@TabPickerSettingActivity,
                                "정보 불러오기 실패 $error",
                                Toast.LENGTH_LONG
                            ).show()
                        } else {
                            val message: String = if (selectedUsers != null) {
                                "정보 불러오기 성공 $selectedUsers"
                            } else {
                                "정보 불러오기 성공 $selectedChat"
                            }
                            SdkLog.i(message)
                            Toast.makeText(
                                this@TabPickerSettingActivity,
                                message,
                                Toast.LENGTH_LONG
                            ).show()
                        }
                    }


                val params = getTabPickerRequestParams()
                if (popupCb.isChecked) {
                    PickerClient.instance.selectPopup(
                        this@TabPickerSettingActivity,
                        params,
                        callback
                    )
                } else {
                    PickerClient.instance.select(
                        this@TabPickerSettingActivity,
                        params,
                        callback
                    )
                }
            }
        }
    }

    private fun ActivityTabPickerSettingBinding.getDisableSelect(): List<DisableSelectOption>? {
        val filter = mutableListOf<DisableSelectOption>()
        if (msgBlockedCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.MSG_BLOCKED))
        if (registeredCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.REGISTERED))
        if (unregisteredCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.UNREGISTERED))
        if (notFriendCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.NOT_FRIEND))
        filter.addAll(disableCustoms)
        return if (filter.isEmpty()) null else filter
    }

    private fun ActivityTabPickerSettingBinding.getPickerOrientation(
        orientationArray: Array<String>
    ) = when (orientationSpinner.selectedItem) {
        orientationArray[1] -> PickerOrientation.AUTO
        orientationArray[2] -> PickerOrientation.PORTRAIT
        orientationArray[3] -> PickerOrientation.LANDSCAPE
        else -> null
    }

    private fun ActivityTabPickerSettingBinding.getViewAppearance(
        viewAppearanceArray: Array<String>
    ) = when (themeSpinner.selectedItem) {
        viewAppearanceArray[1] -> ViewAppearance.AUTO
        viewAppearanceArray[2] -> ViewAppearance.LIGHT
        viewAppearanceArray[3] -> ViewAppearance.DARK
        else -> null
    }

    private fun ActivityTabPickerSettingBinding.getOsFilter(
        deviceOsArray: Array<String>
    ) = when (osSpinner.selectedItem) {
        deviceOsArray[1] -> PickerUsingOsFilter.ALL
        deviceOsArray[2] -> PickerUsingOsFilter.ANDROID
        deviceOsArray[3] -> PickerUsingOsFilter.IOS
        else -> null
    }

    private fun ActivityTabPickerSettingBinding.getPickerFriendFilter(
        friendFilterArray: Array<String>
    ) = when (friendFilterSpinner.selectedItem) {
        friendFilterArray[1] -> PickerFriendFilter.REGISTERED
        friendFilterArray[2] -> PickerFriendFilter.INVITABLE
        friendFilterArray[3] -> PickerFriendFilter.NONE
        else -> null
    }

    private fun ActivityTabPickerSettingBinding.getPickerChatFilter(): List<PickerChatFilter>? {
        val filter = mutableListOf<PickerChatFilter>()
        if (directCb.isChecked) filter.add(PickerChatFilter.DIRECT)
        if (multiCb.isChecked) filter.add(PickerChatFilter.MULTI)
        if (regularCb.isChecked) filter.add(PickerChatFilter.REGULAR)
        if (openCb.isChecked) filter.add(PickerChatFilter.OPEN)
        return if (filter.isEmpty()) null else filter
    }

    private fun ActivityTabPickerSettingBinding.getCountryCodes() =
        if (countryCodeEt.text.isNullOrBlank()) {
            null
        } else {
            countryCodeEt.text.toString().split(',').toList()
        }

    private fun ActivityTabPickerSettingBinding.getMinPickableCount() =
        if (minCountEt.text.toString().isBlank()) {
            minCountEt.hint.toString().toInt()
        } else {
            minCountEt.text.toString().toInt()
        }

    private fun ActivityTabPickerSettingBinding.getMaxPickableCount() =
        if (maxCountEt.text.toString().isBlank()) {
            maxCountEt.hint.toString().toInt()
        } else {
            maxCountEt.text.toString().toInt()
        }

    private fun ActivityTabPickerSettingBinding.getPickerTitle() =
        titleEt.text.toString().ifEmpty {
            null
        }

    private fun getTabPickerRequestParams(): PickerTabRequestParams {
        return binding.run {
            PickerTabRequestParams(
                title = title,
                viewAppearance = viewAppearance,
                orientation = orientation,
                enableSearch = searchbarCb.isChecked,
                disableSelectOptions = disableSelectOptions,
                displayAllProfile = allProfileCb.isChecked,
                maxPickableCount = maxPickableCount,
                minPickableCount = minPickableCount,
                friendsParams = PickerTabRequestParams.FriendsRequestParams(
                    friendFilter = friendFilter,
                    countryCodeFilters = countryCodes,
                    usingOsFilter = osFilter,
                    enableIndex = indexCb.isChecked,
                    showMyProfile = myProfileCb.isChecked,
                    showFavorite = myFavoriteCb.isChecked,
                    showPickedFriend = pickedCb.isChecked
                ),
                chatParams = PickerTabRequestParams.ChatRequestParams(
                    selectionType = if (chatMemberCb.isChecked) PickerChatSelectionType.CHAT_MEMBER else PickerChatSelectionType.CHAT,
                    chatFilters = chatFilter
                )
            )
        }
    }

    companion object {
        const val TAG = "KakaoSDKSample"
    }
}
